/*
		ConstantLoader.cpp: Implementation of the ConstantLoader.cpp class

        Copyright (C) 2000 Patrick Dughi

        This software may be used and distributed according to the terms
        of the GNU Public License, incorporated herein by reference, located
		as the file gpl.txt in this same directory.

        The author may be reached at dughi@imaxx.net.

        Controls handling the reading/generation of ini files, as well as 
		controling the constants it reads in from said source.
*/

#pragma warning(disable:4786)
#include "stdafx.h"
#include "Cbreak.h"
#include "ConstantLoader.h"
#include "utils.h"
#include <map>
#include <string.h>
#include "MapEncap.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

/* this ought to be interesting.. a map of strings to maps of strings */

//map<string, (map<string, int>)> constant_strings;
/* msvc fails again! hurrah!
    Actually, i'm just not sure how to define a template including a reference
	to templates.  It almost seems to work. right before it fails abysmally.
	So, no big deal, i'll just encapsulate it */



#define CONSTANT_FILE		".\\cbreak.ini"

/* this is almost funny */

extern const char *effect_flags[];
extern const char *mact_flags[];
extern const char *maff_flags[];
extern const char *room_flags[];
extern const char *trade_flags[];
extern const char *wear_flags[];

const char *the_constants[] = {
	"Effect Flags",
	"Mob Act Flags",
	"Affect Flags",
	"Room Flags",
	"Trade Flags",
	"Wear Flags",
	"\n",
};

ConstantLoader::ConstantLoader()
{
 
}

ConstantLoader::~ConstantLoader()
{

}

void ConstantLoader::make_new_file()
{
  CircleFile ini_file;
  int i;

   /* so, we have to create one. WritePrivateProfileString makes the ini file
		in windows dir if it can't find it otherwise.  Yes, I could just depend
		on the registry, but, then you couldn't as easily include custom ini
		files for your system */
	ini_file.open_file(fname,1,0);
	if(!ini_file.the_file.good()) {
		error("Error reading ini file.");
		return;
	}
	ini_file.close_file();

	/* i'm sure there's a better way to do this. what it is, I do not know */

	for(i=0;*effect_flags[i] != '\n';i++) {
		WritePrivateProfileString("Effect Flags",effect_flags[i],
				iprintf("%d",1<<i),fname.c_str());
	}

	for(i=0;*mact_flags[i] != '\n';i++) {
		WritePrivateProfileString("Mob Act Flags",mact_flags[i],
				iprintf("%d",1<<i),fname.c_str());
	}

	for(i=0;*maff_flags[i] != '\n';i++) {
		WritePrivateProfileString("Affect Flags",maff_flags[i],
				iprintf("%d",1<<i),fname.c_str());
	}

	for(i=0;*room_flags[i] != '\n';i++) {
		WritePrivateProfileString("Room Flags",room_flags[i],
				iprintf("%d",1<<i),fname.c_str());
	}

	for(i=0;*trade_flags[i] != '\n';i++) {
		WritePrivateProfileString("Trade Flags",trade_flags[i],
				iprintf("%d",1<<i),fname.c_str());
	}

	for(i=0;*wear_flags[i] != '\n';i++) {
		WritePrivateProfileString("Wear Flags",wear_flags[i],
				iprintf("%d",1<<i),fname.c_str());
	}

}

int ConstantLoader::GetSections(string *buf)
{
  CircleFile ini_file;
  string temp;
  MapEncap *newme;
  int i=0;

  buf->erase();
  ini_file.open_file(fname.c_str(),0,0);
 
  do {
		ini_file.get_line(&temp);

	  if(temp.find("[") != -1 && temp.find("]") != -1 && temp.find("=") == -1 ) {
		  i++;
		  newme = new MapEncap;
		  newme->name = temp.substr(temp.find("[")+1,temp.find("]")-1).c_str();
        constant_strings[newme->name]=*newme;
		  *buf += newme->name;
		  *buf += "\n";
		}
	} while(!ini_file.eof());

  ini_file.close_file();
  return i;
}

void ConstantLoader::load_file(string *filename)
{
  CircleFile ini_file;
  int i;
  string temp,temp2;

  fname = *filename;

  if(!ini_file.file_exists(&fname)) {
    make_new_file();
  }

  temp2.erase();
  /* otherwise, it exists, and we have to do something about it.*/ 
  i=GetSections(&temp2);
  if(i <= 0 || !do_required_exist()) {
    make_new_file();
    i=GetSections(&temp2);
  }

  /* now we load */
  LoadKeys();
}

int ConstantLoader::LoadKeys()
{
  CircleFile ini_file;
  string temp,temp2;
  temp2.erase();
  ini_file.open_file(fname.c_str(),0,0);
 
  do {
		ini_file.get_line(&temp);
	  if(temp.find("[") != -1 && temp.find("]") != -1 && temp.find("=") == -1 ) {
      temp2 = temp.substr(temp.find("[")+1,temp.find("]")-1).c_str();
		} else if (!temp2.empty()  && !temp.empty() && !(temp.at(0) == '\n' || temp.at(0) == '\r')) {
      constant_strings[temp2].enmap[temp.substr(0,temp.find("="))]=
				atoi(temp.substr(temp.find("=")+1).c_str());
		}
	} while(!ini_file.eof());

  ini_file.close_file();
  return 1;
}

bool ConstantLoader::do_required_exist()
{
  int i=0;
  string out;

  while(*the_constants[i] != '\n') {
	  out=the_constants[i++];
	  if(constant_strings.find(out) == constant_strings.end()) {
		  return FALSE;
	  }
  }

  return TRUE;
}
